# -*- coding: utf-8 -*-
# @Time    : 2023/4/6 23:38
# @Author  : hxq
# @Software: PyCharm
# @File    : db_helper.py
import contextlib
import re
import json
import importlib
from typing import Union

from hxq.libs.db import DateTimeEncoder
from dbutils.pooled_db import PooledDB, PooledDedicatedDBConnection

__all__ = [
    "DBHelper",
]


class DBHelper:
    """简单的数据库连接池助手"""

    def __init__(self, config):
        self.config = self.import_config(config)
        self.pool = None
        sql_conns = {
            'MySQL': 'pymysql',
            'SQLServer': 'pymssql'
        }
        """
        初始化连接池
        """
        database = importlib.import_module(sql_conns.get(self.config.setdefault('SQL_CREATOR', 'MySQL')))
        self.pool = PooledDB(
            creator=database,
            maxconnections=self.config.setdefault('SQL_MAX_CONNECT', 16),
            mincached=self.config.setdefault('SQL_MIN_CACHED', 1),
            blocking=self.config.setdefault('SQL_BLOCKING', True),
            ping=self.config.setdefault('SQL_PING', 0),
            host=self.config.setdefault('SQL_HOST', '127.0.0.1'),
            port=self.config.setdefault('SQL_PORT', 3306),
            user=self.config.setdefault('SQL_USER', 'root'),
            password=self.config.setdefault('SQL_PASSWORD', 'key123123'),
            database=self.config.setdefault('SQL_DATABASE', None),
            charset=self.config.setdefault('SQL_CHARSET', 'utf8mb4')
        )

    """导入配置"""

    @staticmethod
    def import_config(obj) -> dict:
        """
        导入字典or类对象中的配置信息
        """
        config = dict()
        if isinstance(obj, dict):
            [config.update({k.upper(): v}) for k, v in obj.items()]
        elif hasattr(obj, '__dict__'):
            [config.update({k: getattr(obj, k)}) if k.isupper() else '' for k in dir(obj)]
        else:
            raise ValueError('请导入正确的config对象!')
        return config

    """列表类型转字典"""

    @staticmethod
    def _tuple2dict(results, cursor) -> dict:
        result = [dict(zip([desc[0] for desc in cursor.description], item)) for item in results]
        return json.loads(DateTimeEncoder().encode(result))

    """执行增删改查sql语句"""

    def execute(self, sql_statement: str) -> Union[dict, int]:
        with self.connect() as conn:
            cursor = conn.cursor()
            row_count = cursor.execute(sql_statement)
            # 匹配对大小写不敏感
            if re.search(r"SELECT", sql_statement, re.I):
                result = cursor.fetchall()
                return self._tuple2dict(result, cursor)

            cursor.connection.commit()
            return row_count if row_count else 0

    """查询一条数据"""

    def first(self, sql_statement) -> dict:
        with self.connect() as conn:
            cursor = conn.cursor()
            cursor.execute(sql_statement)
            result = cursor.fetchone()

        return self._tuple2dict([result], cursor)

    """查询所有数据"""

    def all(self, sql_statement) -> dict:
        with self.connect() as conn:
            cursor = conn.cursor()
            cursor.execute(sql_statement)
            result = cursor.fetchall()

        return self._tuple2dict(result, cursor)

    """创建连接"""

    @contextlib.contextmanager
    def connect(self) -> PooledDedicatedDBConnection:
        conn = self.pool.connection()  # 创建连接
        yield conn  # 游标
        self.close(conn)

    """关闭连接"""

    @staticmethod
    def close(conn):
        conn.cursor().close()
        conn.close()

    # def __enter__(self):
    #     """
    #     # 实现with obj as f 主动从连接池中拿连接对象 并存放至线程堆中
    #     """
    #     conn, cursor = self.connect()
    #     return cursor
    #
    # def __exit__(self, exc_type, exc_val, exc_tb):
    #     """
    #     # 实现with 结束 从线程堆中获取链接对象，并回收连接至连接池
    #     """
    #     print(exc_type, exc_val, exc_tb)



# -*- coding: utf-8 -*-
# @Time    : 2022/7/10 20:42
# @Author  : hxq
# @Software: PyCharm
# @File    : common.py
import asyncio
import os
import sys
import pkgutil
import socket
import subprocess
from functools import wraps

from hxq.utils.decorator import except_desc, func_run_times


# 生成资源文件目录访问路径
def resource_path(relative_path=''):
    if getattr(sys, 'frozen', False):  # 是否Bundle Resource
        base_path = getattr(sys, '_MEIPASS')
    else:
        base_path = os.path.abspath(".")  # 获取当前工作路径,
    return os.path.join(base_path, relative_path)


def run_event_loop(func):
    @wraps(func)
    def wrapped(*args, **kwargs):
        return asyncio.get_event_loop().run_until_complete(func(*args, **kwargs))

    return wrapped


@except_desc
def url2ip(url):
    socket_list = socket.getaddrinfo(url, None, 0, socket.SOCK_STREAM)
    ip_info = socket_list[0][4][0]
    print('【{}】 IP地址：{}'.format(url, ip_info))
    return ip_info


@func_run_times()
def array_subtotal(this_list):
    """
    将数组重复数据进行分类汇总，生成一个汇总字典
    """
    data = dict()
    [data.update({item: data[item] + 1}) if item in data else data.update({item: 1}) for item in this_list].clear()
    return data


def run_command(command: str) -> str:
    """
    执行cmd命令
    """
    env = os.environ.copy()
    env["LANG"] = "C"
    output, _ = subprocess.Popen(command,
                                 stdout=subprocess.PIPE,
                                 stderr=subprocess.DEVNULL,
                                 shell=True,
                                 env=env).communicate()
    return output.decode("gbk").rstrip("\r\n")


def get_root_path(import_name: str) -> str:
    """
    通过这个import_name确定Flask这个核心对象被创建的根目录，以获得静态文件和模板文件的目录。
    """
    mod = sys.modules.get(import_name)
    if mod is not None and hasattr(mod, "__file__") and mod.__file__ is not None:
        return os.path.dirname(os.path.abspath(mod.__file__))
    loader = pkgutil.get_loader(import_name)
    if loader is None or import_name == "__main__":
        return os.getcwd()
    if hasattr(loader, "get_filename"):
        filepath = loader.get_filename(import_name)
    else:
        __import__(import_name)
        mod = sys.modules[import_name]
        filepath = getattr(mod, "__file__", None)
        if filepath is None:
            raise RuntimeError(f'找不到{import_name}模块的根路径.')
    return os.path.dirname(os.path.abspath(filepath))


def split_str(text, keys):
    """
    文本多分组分割
    """
    return (lambda _text, _keys: (lambda x:
                                  [(x.update(text=x['text'].replace(k, _keys[0])), x['text'].split(_keys[0]))[1]
                                   for k in _keys if len(x['text'].split(k)) > 1][-1])(dict(text=_text)))(text, keys)


if __name__ == '__main__':
    content = '测试;多组,分割,123.lk'
    split_key = ['.', ',', ';']
    print(split_str(content, split_key), 'text_split 结果')
    print(sys.modules.get('subprocess').__file__)
    print(get_root_path('db'))
    # print(url2ip('z1.m1907.cn'))
    # arr = [1, 3, 5, 7, 9, 5, 1, 3, 6, 7]  # 用于在本地测试的数据，使用时需注释掉前两行
    # print(array_subtotal(arr))
    # for i in range(1, 10000):
    #     arr.append(random.randint(1, 99999))

    # print(run_command('ipconfig'))



# -*- coding: utf-8 -*-
# @Time    : 2023/4/18 23:00
# @Author  : hxq
# @Software: PyCharm
# @File    : main.py
from hxq.interface.db_helper import DBHelper
from hxq.interface.httpx import HTTP, http

# if __name__ == '__main__':
#     CONFIG = {
#         'SQL_CREATOR': 'MySQL',
#         'SQL_HOST': '127.0.0.1',
#         'SQL_USER': 'root',
#         'SQL_PASSWORD': '123456',
#         'SQL_DATABASE': 'blog'
#     }
#     db = DBHelper(config=CONFIG)
#     print(db.all("SHOW DATABASES;"))
#     print(db.first("SELECT * FROM  rule_group"))

if __name__ == '__main__':
    HTTP().download(
        'https://infinitypro-img.infinitynewtab.com/wallpaper/travel/5596.jpg?attname=infinity-19779309.jpg'
    )



# -*- coding: utf-8 -*-
# @Time    : 2022/7/9 22:21
# @Author  : hxq
# @Software: PyCharm
# @File    : ran_func.py
import string
import random


def random_password(length=12):
    """
    随机密码生成
    """
    return ''.join(random.sample(string.digits + string.ascii_letters * 1, length))



# -*- coding: utf-8 -*-
# @Time    : 2022/6/9 22:03
# @Author  : hxq
# @Software: PyCharm
# @File    : 多线程.py
# -*- coding: utf-8 -*-
import ctypes
import inspect
import threading
from threading import Thread, Timer, enumerate, Event, RLock


# 定时器(每N秒执行一次)
class TimerAlways(object):
    def __init__(self, func, time=1.0, daemon=True, *args, **kwargs):
        self.func = func
        self.time = time
        self.daemon = daemon
        self.args = args
        self.kwargs = kwargs
        self.__timer = None

    # 开启
    def start(self):
        print('{} 秒后开始执行'.format(self.time))
        self.__set_wall_timer()

    # 定时等待循环执行任务
    def __set_wall_timer(self):
        self.func(*self.args, **self.kwargs)
        self.__timer = Timer(self.time, self.__set_wall_timer)
        self.__timer.setDaemon(self.daemon)
        self.__timer.start()

    # def start_timer(self, seconds, callback):
    #     def timer():
    #         threading.Timer(seconds, timer).start()
    #         callback()
    #
    #     timer()
    # 终止定时
    def exit(self):
        self.__timer.cancel()


# 多线程操作
class ThreadX(Thread):
    lock = RLock()

    def __init__(self, func: callable, *args, **kwargs):
        daemon = kwargs.pop("daemon", False)
        super().__init__(target=func, args=args, kwargs=kwargs, name=func.__name__)
        self.setDaemon(daemon)
        self.start()
        self.__flag = Event()  # 用于暂停线程的标识

    def pause(self):
        """
        线程阻塞
        """
        self.__flag.clear()

    def resume(self):
        """
        阻塞
        """
        self.__flag.set()

    def stop_thread(self):
        """
        终止线程
        """
        if self.ident:
            self._async_raise(self.ident, SystemExit)

    @classmethod
    def show_time(cls):
        """
        显示当前线程数量
        """
        length = len(enumerate())  # 枚举返回个列表
        print('当前运行的线程数为：%d' % length)
        # 当发现子线程没结束时，会增加一个线程，如果让在线程最后，那么就不会新增一个线程
        timer = Timer(3, cls.show_time)
        timer.setDaemon(True)
        timer.start()

    @staticmethod
    def _async_raise(tid, exctype):
        """引发异常，如果需要，执行清理"""
        tid = ctypes.c_long(tid)
        if not inspect.isclass(exctype):
            exctype = type(exctype)
        res = ctypes.pythonapi.PyThreadState_SetAsyncExc(tid, ctypes.py_object(exctype))
        if res == 0:
            # 触发异常
            raise ValueError("无效的线程ID")
        elif res != 1:
            # """if it returns a number greater than one, you're in trouble,
            # and you should call it again with exc=NULL to revert the effect"""
            ctypes.pythonapi.PyThreadState_SetAsyncExc(tid, None)
            raise SystemError("PyThreadState_SetAsyncExc failed")


class ThreadManager:
    def __init__(self):
        self.threads = []

    def add_thread(self, target, args=()):
        thread = threading.Thread(target=target, args=args)
        self.threads.append(thread)
        thread.start()

    def wait_for_threads(self):
        for thread in self.threads:
            thread.join()



from setuptools import find_packages, setup

setup(name='hxq',
      author='hxq',
      version='1.0.2',
      packages=find_packages(exclude=["*.demo", "*.demo.*", "tests", "demos"]),
      author_email='337168530@qq.com',
      description="这是一个python工具包",
      license="GPL",
      # 而 extras_require 这里仅表示该模块会依赖这些包,深度使用模块时，才会用到，这里需要你手动安装
      extras_require={
          'HTML': ["bs4>=0.0.1", "xmltodict>=1.2"],
      },
      # install_requires 在安装模块时会自动安装依赖包
      install_requires=[
          'requests>=2.24.0',
          'lxml>=4.9.2',
          'pymysql~=1.0.3',
          'DBUtils~=3.0.2',
          'aiofiles~=23.1.0',
      ]
      )



# -*- coding: utf-8 -*-
# @Time    : 2022/5/7 21:47
# @Author  : hxq
# @Software: PyCharm
# @File    : decorator.py
import time
from datetime import datetime
from functools import wraps


def func_run_times(length: int = 3):
    """
    函数运行耗时装饰器
    :param length:默认3位小数
    :return:
    """
    """
    # @functools.wraps(func),这是python提供的装饰器。
    # 它能把原函数的元信息拷贝到装饰器里面的 func 函数中。函数的元信息包括docstring,name,参数列表等等
    # 可以尝试去除@functools.wraps(func),你会发现"func".__name__的输出变成了inter
    # 经过装饰后实质上是得到的函数体是wrapper
    @func_run_times()
    def a():
        ...
    print(a.__name__)
    """

    def wrapper(fn):
        def print_red(string):
            print(f"\033[36m{string}\033[0m")

        @wraps(fn)
        def inter(*args, **kwargs):
            start = time.time()
            res = fn(*args, **kwargs)
            print_red("run time of the func is '%s' %.*f Ms" % (fn.__name__, length, (time.time() - start) * 1000))
            return res

        return inter

    return wrapper


# 统一的异常输出
def except_desc(func):
    @wraps(func)
    def inter(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception as e:
            print('=' * 30)
            print('程序出现了异常')
            print(f'----异常时间：\t{datetime.now()}\n----异常函数：\t{func.__name__}\n----异常原因：\t{e}')
            print('=' * 30)
            return e

    return inter



# -*- coding: utf-8 -*-
# @Time    : 2022/12/6 23:18
# @Author  : hxq
# @Software: PyCharm
# @File    : __init__.py
import datetime
from json import JSONEncoder


class DateTimeEncoder(JSONEncoder):
    def default(self, obj):
        if isinstance(obj, (datetime.date, datetime.datetime)):
            return obj.isoformat(sep=" ")
        return super().default(obj)



# -*- coding: utf-8 -*-
# @Time    : 2022/5/7 22:22
# @Author  : hxq
# @Software: PyCharm
# @File    : http.py
import importlib
import sys
import json
from functools import wraps
from typing import Union, List
from lxml import etree, html
from requests.packages import urllib3
from requests import Session, Response, exceptions
from requests.cookies import cookiejar_from_dict, RequestsCookieJar
from hxq.utils.common import run_event_loop

if sys.version_info >= (3,):
    from urllib.parse import urlencode, urlparse
    from urllib import parse
else:
    from urllib import urlencode, parse
# 关闭错误提示
urllib3.disable_warnings()


@run_event_loop
async def iter_write_file(file_path: str, r: Response):
    import aiofiles
    async with aiofiles.open(file_path, 'wb') as file:
        for chunk in r.iter_content(1024 * 10):
            await file.write(chunk)


def requests_catch_exception(func):
    """
    requests异常捕获装饰器
    """

    @wraps(func)
    def wrapper(*args, **kwargs) -> Union[HTTPResponse, HTTPError]:
        try:
            return func(*args, **kwargs)
        except exceptions.HTTPError:
            return HTTPError(1000, 'HTTP错误!')
        except exceptions.SSLError:
            return HTTPError(1001, 'SSL错误!')
        except exceptions.ProxyError:
            return HTTPError(1002, '代理错误!')
        except exceptions.ConnectTimeout:
            return HTTPError(1003, '请求响应超时!')
        except exceptions.ConnectionError:
            return HTTPError(1004, '网络连接错误!')
        except exceptions.ChunkedEncodingError:
            return HTTPError(1005, 'chunked编码错误!')
        except exceptions.ContentDecodingError:
            return HTTPError(1006, '内部解码错误!')
        except Exception as e:
            return HTTPError(1007, e)

    return wrapper


class HTTPResponse:

    def __init__(self, response: Response):
        """
        :param response:http响应请求体
        """
        self.ok = True
        self.__html = None
        self.__response = response
        self.code = response.status_code
        if response.encoding and response.encoding.lower() in ['iso-8859-1', 'gb2312']:
            """
            部分页面没有编码"说明"，无法检测到编码类型,如果默认编码在常用编码类型之外将强制转化为最常见的gbk编码
            也可手动更改或使用apparent_encoding做自适应
            虽然默认编码是GB2312,但是实际上使用的是GBK编码 且 GB2312 是 GBK子集
            """
            self.encoding = 'GBK'

    def apparent_encoding(self):
        """
        应用自适应检测网页编码,会大幅度降低速度且不能做到完全匹配
        """
        self.encoding = self.__response.apparent_encoding
        if self.encoding.lower() == 'gb2312':
            self.encoding = 'GBK'
        return self.encoding

    @property
    def encoding(self):
        return self.__response.encoding

    @encoding.setter
    def encoding(self, encode):
        self.__response.encoding = encode

    @property
    def json(self) -> dict:
        if (3,) <= sys.version_info < (3, 6):
            return json.loads(self.res.content.decode(self.encoding), encoding=self.encoding)
        return self.__response.json()

    @property
    def lxml(self):
        # 按需install bs4
        """
        使用难度最低,解析速度最慢
        @return:
        """
        bs4 = importlib.import_module('bs4')
        return getattr(bs4, 'BeautifulSoup')(self.__response.text, "lxml")

    def xpath(self, *args) -> List[html.HtmlElement]:
        """
        使用难度中等，解析速度中等
        @param args:
        @return:
        """
        self.__html = self.__html if self.__html is not None else etree.HTML(self.text)
        return self.__html.xpath(*args)

    @property
    def text(self):
        return self.__response.text

    @property
    def content(self):
        return self.__response.content

    @property
    def res(self):
        return self.__response

    @property
    def cookies(self):
        return HTTPRequest.cookiejar2dict(self.__response.cookies)

    @property
    def headers(self):
        return self.__response.headers

    def xml2dict(self, xml_data=None) -> dict:
        xml_to_dict = importlib.import_module('xmltodict')
        # 按需安装
        _parse = getattr(xml_to_dict, 'parse')
        dict_xml = _parse(xml_data) if xml_data else _parse(self.text.encode("utf-8"))
        return json.loads(json.dumps(dict_xml))

    def dict2xml(self, json_str: Union[dict, str] = None):
        json_str = json_str if json_str else self.json
        if isinstance(json_str, str):
            json_str = json.loads(json_str)
        if len(json_str.keys()) > 1:
            json_str = {'root': json_str}

        xml_to_dict = importlib.import_module('xmltodict')
        _unparse = getattr(xml_to_dict, 'unparse')
        return _unparse(json_str, pretty=1)

    def close(self):
        # print('链接关闭')
        self.res.close()

    def __str__(self):
        return "{}:{}".format(self.__response, self.encoding)


class HTTPError:
    """请求错误"""

    def __init__(self, code, error_reason=None):
        """
        :param code: http状态码
        :param error_reason: http状态原因
        """
        self.ok = False
        self.code = code
        self.error_reason = error_reason

    def close(self):
        pass
        # self.res.close()

    def __str__(self):
        return "HTTP {}: {}".format(self.code, self.error_reason)


class HTTPRequest:

    @classmethod
    def get(cls, url, params=None, timeout=None, proxies=None, **kwargs) -> Union[HTTPResponse, HTTPError]:
        """
        GET请求
        """
        kwargs.setdefault('allow_redirects', True)
        r = cls.request('GET', url, params=params, timeout=timeout, proxies=proxies, **kwargs)
        r.close()
        return r

    @classmethod
    def post(cls, url, data=None, timeout=None, proxies=None, **kwargs) -> Union[HTTPResponse, HTTPError]:
        """
        POST请求
        """
        r = cls.request('POST', url, data=data, timeout=timeout, proxies=proxies, **kwargs)
        r.close()
        return r

    @classmethod
    def download(cls, url, file_path=None, params=None, data=None, **kwargs):
        """
        下载文件
        """
        kwargs['stream'] = True
        request = kwargs.pop('request', cls.request)
        r = request(kwargs.pop('method', "get").upper(), url, params=params, data=data, **kwargs)
        if not r.ok:
            return r.error_reason
        if not file_path:
            file_path = urlparse(r.res.url).path.split('/')[-1]
        iter_write_file(file_path, r.res)
        r.close()
        return True

    @staticmethod
    @requests_catch_exception
    def request(method, url, **kwargs):
        """
        :param method: 大小不敏感，在Session内部upper统一转化为大写
        :param url: 地址
        :param kwargs:
        :return:
        """

        kwargs.setdefault('ignore_status', True)
        ignore_status = kwargs.pop('ignore_status')
        ua = "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/112.0.0.0 " \
             "Safari/537.36"
        kwargs.setdefault('headers', {"User-Agent": ua})
        results = HTTPResponse(Session().request(method, url, **kwargs))
        if not ignore_status and results.code != 200:
            return HTTPError(results.code, '请求状态错异常!')
        return results

    @staticmethod
    def cookiejar2dict(cookiejar: RequestsCookieJar) -> dict:
        """
        CookieJar转dict
        """
        return (lambda ck: [[ck.update({k: v}) for k, v in cookiejar.items()], ck][1])(dict())

    @staticmethod
    def dict2cookiejar(cookie_dict: dict) -> RequestsCookieJar:
        """
        dict转CookieJar
        """
        return cookiejar_from_dict(cookie_dict)

    @staticmethod
    def _raw_headers_to_dict(this_headers_str: str) -> dict:
        """
        抓包raw字符串解析为headers
        """
        # lambda
        return (lambda x: [[(lambda xx: (
            lambda items: [xx.update({items[0].strip(): ':'.join(items[1:]).strip()}), xx][1]))(x)(
            item.strip().split(':')) for item in this_headers_str.strip().split('\n') if item.strip()], x])(dict())[1]

    @staticmethod
    def dict2url(dict_params) -> str:
        """
        字典转url
        """
        return urlencode(dict_params)

    @staticmethod
    def quote(content, encoding='utf-8', **kwargs):
        """
        url编码
        """
        return parse.quote(content, encoding=encoding, **kwargs)

    @staticmethod
    def unquote(content, encoding='utf-8', **kwargs):
        """
        url解码
        """
        return parse.unquote(content, encoding=encoding, **kwargs)
